/*
	NCP-010 Controller Version 11/19/2010-A
	(C) 2010 Jason Hunt nulluser@gmail.com

	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	File: analog.c
*/


#include "core.h"

unsigned int analog[NUM_ANALOG];


unsigned char cur_analog = 0;

const unsigned char chan_seq[] = {0x0c, 0x0b};
	
unsigned long sample_count = 0;
unsigned long sample_rate = 0;


#define SAMP_PER_INTR  16





/* This will fire every 10 milliseconds */
void __attribute__((__interrupt__, auto_psv)) _ADCInterrupt (void)
{
	unsigned long tmp = 0, i;

	unsigned int *p = (unsigned int *)&ADCBUF0;


	for (i = 0; i < SAMP_PER_INTR; i++)
		tmp += *(p++);

	//cur_analog = 0;

	sample_count += SAMP_PER_INTR;

	analog[cur_analog] = tmp/SAMP_PER_INTR; // ADCBUF0 +; // Get analog data from buffer

	cur_analog++;


	// Compute Next channel to sample
	if (cur_analog > 2) cur_analog = 0;

	ADCHSbits.CH0SA = chan_seq[cur_analog];

	IFS0bits.ADIF = 0;

//	ADCON1bits.SAMP = 1; // start sampling then ...

}
/* End of _ADCInterrupt */


/* Setup ADC */
void analog_init ( void )
{
	//TRISB = 0x1000;

	TRISB = 0b0001100000000000; // RB11, RB12 = Inputs, others: Outputs
	ADPCFG = 0b1110011111111111; // RB11, RB12 = analog


	cur_analog = 0;


	ADCSSL = 0;


	ADCON1bits.ASAM = 1;
	ADCON1bits.SSRC = 7;

	ADCON2bits.SMPI = SAMP_PER_INTR-1; // samples per interrupt

	ADCON3bits.SAMC = 31;
	ADCON3bits.ADCS = 31;//13;//6;     

	// Fcy = 20000000
	// Tcy = 50 nS
	// ADCS = 31
	// Tad = 800nS
	// 
	


	ADCHSbits.CH0SA = chan_seq[cur_analog];

	IEC0bits.ADIE = 1;

	ADCON1bits.ADON = 1; // turn ADC ON


//	ADCON1bits.SAMP = 1; // start sampling then ...

}
/* End of analog_init */


