/*
	NCP-010 Controller Version 11/19/2010-A
	(C) 2010 Jason Hunt nulluser@gmail.com

	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	File: main.c
*/

#include "core.h"



#define SERIAL_CAM_HEADER_SIZE      5
#define SERIAL_CAM_FOOTER_SIZE      5


#define CAMERA_BLK	192

int msg_camera = 0;


unsigned int camera_mode = 10;		// Set baud mode

unsigned int camera_delay = 100;



unsigned int camera_size = 0;
unsigned int camera_addr = 0;


#define CAM_BUFF_SIZE CAMERA_BLK + 10	

unsigned char camera_buff[CAM_BUFF_SIZE];
unsigned char tb[CAMERA_BLK + 10];

void camera_write(unsigned char c);
unsigned char camera_read(unsigned char *data);
void camera_packet(unsigned char *d, unsigned int len);

int serial_transfer(unsigned char *t, unsigned int t_len, 
			unsigned char *r, unsigned int r_len);





/* Write a byte to the camera bus */
void camera_write(unsigned char c)
{
	while (U2STAbits.UTXBF);
	U2TXREG  = c;
	
}
/* End of camera write */


/* Read a byte from the camera bus */
unsigned char camera_read(unsigned char *data)
{

	unsigned long timeout = 1000000;

	while (!U2STAbits.URXDA && --timeout);

	if (timeout == 0) return(1);

//		return(U2RXREG & 0xff);;

	*data = U2RXREG & 0xff;


	return(0);
}
/* End of camera_read */



/* Send a packet to the camera */
void camera_packet(unsigned char *d, unsigned int len)
{
	unsigned int i;

	for (i = 0; i < len; i++)
		camera_write(d[i]);

}
/* End of camera_packet */


/* Send a packet to the bus and wait for the responce */
int serial_transfer(unsigned char *t, unsigned int t_len, 
			unsigned char *r, unsigned int r_len)
{
	camera_packet(t, t_len);
	unsigned int i;

//	unsigned char data[camera_blk+16];

	unsigned char timeout = 0;;

	for (i = 0; i < r_len && !timeout; i++)
		timeout = camera_read(r + i);

	if (timeout) return(1);


//	unsigned char tx[128];

//		debug("rx:");
	//for (i = 0; i < r_len; i++)
		//hex_str(tx + i*2, r[i]);

//	debug((char*)tx);

	return(0);
}




/* Send camera message to host */
void camera_send_message( unsigned char message )
{
	unsigned int i;

	unsigned char tx_buff[6];

	tx_buff[0] = 5;				// Total packet size
	tx_buff[1] = DEVICE_CAM1;		// Device ID
	tx_buff[2] = 7;				// Image size packet
	tx_buff[3] = message;
	tx_buff[4] = get_checksum(tx_buff, 4);

	for (i = 0; i < 5; i++)
		serial_host_tx(tx_buff[i]);

}
/* End of camera_send_message */


/* Send size message to host */
int camera_send_size( void )
{
	unsigned char tx_buff[6];

	tx_buff[0] = 6;				// Total packet size
	tx_buff[1] = DEVICE_CAM1;		// Device ID
	tx_buff[2] = 5;				// Image size packet
	tx_buff[3] = camera_size >> 8;
	tx_buff[4] = camera_size & 0xff;
	tx_buff[5] = get_checksum(tx_buff, 5);

	unsigned int i;

	for (i = 0; i < 6; i++)
		serial_host_tx(tx_buff[i]);


	//timer_delay(100);
	return(0);
}
/* End of camera_send_size */


/* Send data block to host */
int camera_send_block( void )
{
	unsigned int i,j;
	
	i = 0;

	tb[i++] = CAMERA_BLK+4;			// Total packet size
	tb[i++] = 0x8;			// Device Number
	tb[i++] = 6;			// image data parameter

	for (j = 0; j < CAMERA_BLK; j++)
		tb[i++] = camera_buff[j+5];  // +5 to skip over header

	tb[i] = get_checksum(tb, i);


	for (i = 0; i < CAMERA_BLK+4; i++)
		serial_host_tx(tb[i]);

	return(0);

}
/* End of camera_send_block */



/* Send get image command */
int camera_start_image(void)
{
	unsigned char data[] = {0x56, 0x00, 0x36, 0x01, 0x00};

	if(serial_transfer(data, 5, data, 5)) return(1);


	#if 0
	camera_send_message(data[0]);
	camera_send_message(data[1]);
	camera_send_message(data[2]);
	camera_send_message(data[3]);
	camera_send_message(data[4]);
	#endif

	return(0);
}
/* End of camera_get_image */


/* Send stop image command */
int camera_stop_image()
{
   
    unsigned char data[] = {0x56, 0x00, 0x36, 0x01, 0x03};
    
    if (serial_transfer(data, 5, data, 5)) return(1);
 

    return(0);
}
/* End of camera_stop_image */


/* Send read block command */
int camera_get_block( void )
{
 	unsigned char interval = 0x0a;

	unsigned char cmd[] = {0x56, 0x00, 0x32, 0x0c, 0x00, 0x0a, 0x00, 0x00,  
        			camera_addr >> 8, camera_addr & 0xff, 0x00, 0x00, 
				CAMERA_BLK >> 8,  CAMERA_BLK & 0xff, 0x00, interval};

	// send command and get header
	if (serial_transfer(cmd, sizeof(cmd), camera_buff, CAMERA_BLK + 10)) return(1);

    	camera_addr += CAMERA_BLK;                  
	//Sleep(200);

	return(0);
}
/* End of camear_get_block */


/* Send Get JPEG filesize command */
int camera_get_filesize()
{
	unsigned char data[] = {0x56, 0x00, 0x34, 0x01, 0x00, 0, 0, 0, 0};
    
	if (serial_transfer(data, 5, data, 9))
		return(1);

	camera_size = (data[7] << 8) + data[8];

	return(0);
}
/* End of camera_get_filesize*/



/* Send set size command */
int camera_set_size( int image_size )
{
  /*  log(MODULE_NAME " [%s - %d] - Set Size", name, device_id);

    // Size command
    unsigned char cmd_160_120[] = {0x56,0x00 ,0x31, 0x05, 0x04, 0x01, 0x00, 0x19, 0x22};
    unsigned char cmd_320_240[] = {0x56,0x00 ,0x31, 0x05, 0x04, 0x01, 0x00, 0x19, 0x11};
    unsigned char cmd_640_480[] = {0x56,0x00 ,0x31, 0x05, 0x04, 0x01, 0x00, 0x19, 0x00};

    unsigned char data[5];

    int ret = 1;
    
    if (image_size == SERIAL_CAM_SIZE_160_120)
    {
        ret = serial.serial_transfer(device_id, cmd_160_120, sizeof(cmd_160_120), data, 5);
    } else
    if (image_size == SERIAL_CAM_SIZE_320_240)
    {
        ret = serial.serial_transfer(device_id, cmd_320_240, sizeof(cmd_320_240), data, 5);
    } else
    if (image_size == SERIAL_CAM_SIZE_640_480)
    {
        ret = serial.serial_transfer(device_id, cmd_640_480, sizeof(cmd_640_480), data, 5);
    } 

    camera_reset();
    Sleep(200);

    return(ret);*/
	return(0);

}
/* End of camera_set_size */


/* Send set baud command */
int camera_set_baud( void )
{
	return(0);

	// Compute divisor for camera baud rate
	unsigned long camera_baud = 125000;//78125;
	unsigned int divisor =  432000000 / camera_baud - 256;

	unsigned char data[] = {0x56, 0x00, 0x24, 0x03, 0x01, divisor>>8,divisor&0xff};

   
	if (serial_transfer(data, 7, data, 5))
		return(1);


	timer_delay(10);

	camera_send_message(11);


	U2MODEbits.UARTEN = 0;
	U2STAbits.UTXEN = 0;


	U2BRG = (((FCY / camera_baud) / 16) - 1);		// Create and set baud counter

	U2MODEbits.UARTEN = 1;
	U2STAbits.UTXEN = 1;

	timer_delay(10);


	camera_send_message(12);


    return(0);
}
/* End of camera_set_baud */


/*  Reset the camera */
void camera_hw_reset()
{
/*    log(MODULE_NAME " [%s - %d] - Reset", name, device_id);
 
    unsigned char cmd[] = {0x56, 0x00, 0x26, 0x00};
    unsigned char data[64];
 
    bool init_ok = false;

    while (!init_ok)
    {
        if (serial.serial_transfer(device_id, cmd, 4, data, 71) == 0)
            init_ok = 1;
           
        Sleep(2000);                             
    }    */
  
   // flush_serial();

//    stop_serial();
//    start_serial(CBR_38400);

//    camera_set_baud();
 //   Sleep(200);

    //log(MODULE_NAME " [%s - %d] - Init OK", name, device_id);

}
/* End of camera_reset */



/* Get an image from the camera */
int camera_get_image( unsigned char *buff, unsigned int *image_size )
{
  /*  log(MODULE_NAME " [%s - %d] - Main Get Image", name, device_id);
    return(image_error);
*/
return(0);
}
/* End of camera_get_image */


/* The camera thread */
int camera_thread( )
{
  /*  log(MODULE_NAME " [%s - %d] - Start Thread", name, device_id);
    //camera_set_baud(); // Set fast baud rate
   //  camera_set_size(image_format);  // Resize image abd buffers
    camera_setup_image(image_format);
*/
        
    return(0);
}
/* End of camera thread */


/* Reset camera */
void camera_reset( void )
{
	camera_mode = 4;
	camera_stop_image();	
}
/* End of camera reset */


/* Process camera state */
int camera_state( void )
{

	if (camera_mode == 0)
	{	
		camera_send_message(0);

		if (camera_start_image()) return(1);
		
		camera_mode = 1;
		camera_delay = 15;
		camera_addr = 0;


	} else

	if (camera_mode == 1)
	{
		if (camera_get_filesize()) return(1);

		if (camera_send_size()) return(1);

		if (camera_size == 0)
		{
			camera_mode = 0;
			camera_delay = 0;
		}

//		camera_count = 1;
		camera_mode = 2;

//		camera_send_message(1);
		camera_delay = 10;
	} else

	if (camera_mode == 2)
	{
		if (camera_get_block()) return(1);
		if (camera_send_block()) return(1);

		if (camera_addr >= camera_size)
			camera_mode = 3;
	
//		camera_send_message(2);
		camera_delay = 0;


	} else

	if (camera_mode == 3)
	{
		if (camera_stop_image())return(1);
		camera_mode = 4;
//		camera_send_message(3);
		camera_delay = 5;
	} else

	if (camera_mode == 4)
	{
		camera_mode = 0;
//		camera_send_message(4);
		camera_delay = 5;

	} else
	if (camera_mode == 10)
	{
		camera_mode = 0;
//		camera_send_message(10);
		camera_delay = 10;
		camera_set_baud();

	} else
	
		camera_mode = 4;


	return(0);


}
/* End of  camera_state */



/* Update Camera */
void camera_update(void)
{

	if (camera_delay) return;

//	camera_send_message(20);
//return;

	if (host_rx_error) return; 



	if (camera_state())
	{
		camera_mode = 0;
		camera_delay = 10;
	}


}
/* End of update camera */



