/*
	NCP-010 Controller Version 11/19/2010-A
	(C) 2010 Jason Hunt nulluser@gmail.com

	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "core.h"

#define RX_SIZE       8 
#define NUM_SERVOS	4	   		    // Number of servo outputs

unsigned char servo_mode = 0;			// Current state of the servo interrupt system
unsigned char cur_servo = 0;			// Current servo 

unsigned char servo_pos[NUM_SERVOS];

char servo_trim[NUM_SERVOS];

unsigned int tele_send = 0;

int msg_telemetry = 0;
int msg_controller = 0;


unsigned char debug_buffer[300];
unsigned char debug_len = 0;
unsigned char msg_debug = 0;


/* Send debug string */
void debug(char *c)
{
//serial_str("ED\n");

	if (host_rx_error) return; 
	unsigned int i = 0, j=0;

	i++;					// Total packet size
	debug_buffer[i++] = 15;			// Device ID

	while(c[j])
		debug_buffer[i++] = c[j++];

	debug_buffer[0] = i+1;

	debug_buffer[i] = get_checksum(debug_buffer, i);
	

	debug_len = i+1;


	msg_debug = 1;
//serial_str("LD\n");
}
/* End of debug */


/* Convert decimal into hext for one nibble */
unsigned char hex_char( unsigned char n)
{
	if (n >= 10)
		return(n - 10 + 'A');
	else
		return(n + '0');
}
/* End of hext char */


/* Make a hex string */
void hex_str( unsigned char *s, unsigned char v)
{
	s[0] = hex_char(v >> 4);
	s[1] = hex_char(v & 0x0f);
	s[2] = 0;
}
/* End of hex str */


/* Send debug message */
void transfer_debug( void )
{
//serial_str("ET\n");

	if (host_rx_error) return; 

	while(host_tx_pending)
	{
		///serial_hex(host_tx_pending);
	}

//serial_str("WT\n");

	unsigned int i = 0;

	while(i < debug_len)
	{
		serial_host_tx(debug_buffer[i]);
		i++;
	}

	msg_debug =0;
	while(host_tx_pending)
	{
		///serial_hex(host_tx_pending);
	}
//serial_str("DT\n");

}
/* End of transfer debug */


/* Update the servos */
void update_servoes( void )
{
	if (servo_mode == 0) // setupup for base pos wait
	{
		servo_mode = 1; 					   // next trigger we will setup for base delay

		if (cur_servo == 0 && servo_pos[0] != 0x80) _LATF0 = 1; else
		if (cur_servo == 1 && servo_pos[1] != 0x80) _LATF1 = 1; else
		if (cur_servo == 2 && servo_pos[2] != 0x80) _LATF6 = 1; else
		if (cur_servo == 3 && servo_pos[3] != 0x80) _LATD0 = 1; 

		PR2 = 2550 + (unsigned int)(servo_pos[cur_servo] + servo_trim[cur_servo])* 10;

	} else
	if (servo_mode == 1)					// setupfor next servo
	{
		servo_mode = 0;						// reset back to base mode

		if (cur_servo == 0) _LATF0 = 0; else
		if (cur_servo == 1) _LATF1 = 0; else
		if (cur_servo == 2) _LATF6 = 0; else
		if (cur_servo == 3) _LATD0 = 0; 

		cur_servo++; // next servo
		
		// Setup longer delay to fill the 20 ms window, 
		// we need to burn 11ms on average
		// 5Mhz clock, 0.0512 MS per increment at 1:256 prescaler
		// 11ms / 0.0512 ms = 215 cycles to burn, start time at 40

		if (cur_servo == NUM_SERVOS)
		{
			cur_servo = 0;	// reset back the first servo
			PR2 = 45000;
		}
	}
}
/* End of void update_servoes */



/* Send controller telemetry */
void controller_telemetry( void )
{

	// Check for space in tx buffer
	//if (MAX_HOST_TX - host_tx_pending < 16) return;
	//debug("St\n");

	if (host_rx_error) return; 

	while(host_tx_pending)
	{
		///serial_hex(host_tx_pending);
	}

//	if (host_tx_pending > 0) return;
//	debug("Send telemetry\n");



//	if (sending) return;

	static unsigned char tx_buff[16];

	tx_buff[0] = 16;			// Total packet size
	tx_buff[1] =  DEVICE_CONT;		// Device ID
	tx_buff[2] = 0x11;//analog[0] & 0xff;
	tx_buff[3] = 0x22;//analog[1] & 0xff;

	tx_buff[4] = 0x33;//(analog[0] >> 8)  | ((analog[1]&0xf00) >> 4);
	tx_buff[5] = 0x44;
	tx_buff[6] = 0x55;
	tx_buff[7] = 0x66;

	tx_buff[8] = 0x77;//(analog[0] >> 8)  | ((analog[1]&0xf00) >> 4);
	tx_buff[9] = 0x88;
	tx_buff[10] = 0x99;
	tx_buff[11] = 0xaa;

	tx_buff[12] = 0xbb;//(analog[0] >> 8)  | ((analog[1]&0xf00) >> 4);
	tx_buff[13] = 0xcc;
	tx_buff[14] = 0xdd;
	tx_buff[15] = get_checksum(tx_buff, 15);



	unsigned int i;

	for (i = 0; i < 16; i++)
		serial_host_tx(tx_buff[i]);

	msg_telemetry = 0;


}
/* End of controller_telemetry */



/* Deal with control packet */
void controller_packet( unsigned char *buff )
{
	update_timeout = 0;


	servo_pos[0] = buff[0];
	servo_pos[1] = buff[1];
	servo_pos[2] = buff[2];
	servo_pos[3] = buff[3];

//	serial_str("DP");

//	serial_hex( buff[0]);
//	serial_hex( buff[1]);
//	serial_hex( buff[2]);
//	serial_hex( buff[3]);


//	serial_str("\n");

}
/* End of controller_packet */




/* Update controller */
void controller_check( void )
{
	if (!host_rx_error)
	{	
		error_act =0;

		if (++tele_send >= 10)
		{
			msg_telemetry = 1;
			tele_send = 0;
		}

	}
	else
		error_act = 1;


}
/* End of controller_check */


/* Reset the controller */
void controller_reset( void )
{
	servo_pos[0] = 0x80;
	servo_pos[1] = 0x80;
	servo_pos[2] = 0x80;
	servo_pos[3] = 0x80;

}
/* End of void controller_reset */



/* Setup controller data */
void controller_init ( void )
{
	controller_reset();

	servo_trim[0] = 6;
	servo_trim[1] = 0;
	servo_trim[2] = -6;
	servo_trim[3] = 6;
}
/* End of controller_init */


/* update controller */
void controller_process( void )
{


}
/* End of controller process */


