/*
	NCP-010 Controller Version 11/19/2010-A
	(C) 2010 Jason Hunt nulluser@gmail.com

	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	File: serial.c
*/

#include "core.h"
#define BUFFER_HOST_TX	1			// Enable buffered tx to host


#define MODE_WAIT	0
#define MODE_SIZE	1
#define MODE_DATA	2
#define MODE_CHECK	3


#define HOST_BAUD	250000			// Baud for host pc
#define U2_BAUD		38400			// Baud for host pc



#define MAX_UPDATE_TIME 50			// 1/100 sec. Time before resetting serial

#define U2_TX_SIZE	300
#define MAX_HOST_TX	256
#define RX_BUFF_SIZE	240


unsigned int host_rx_error = 1;			// Set inital error to force reset


unsigned char serial_mode = MODE_WAIT;		// CUrrent FSM mode for serial
unsigned char packet_size;			// Size of this packet
unsigned char device_id = 0;			// CUrrent device id
unsigned char data_pending = 0;			// Data remaining in packet


unsigned int update_timeout = 0;		// Timeout counter. Reset when > max


unsigned char u2_tx_buff[U2_TX_SIZE];
int u2_tx_head = 0;
int u2_tx_tail = 0;


// Buffering for controller to host data
unsigned char host_tx_buff[MAX_HOST_TX];
int host_tx_tail = 0;
int host_tx_head = 0;
int host_tx_pending = 0;


unsigned char rx_buff[RX_BUFF_SIZE];
unsigned char cur_rx = 0;



/*

void serial_ch(unsigned char c)
{
	while (U2STAbits.UTXBF);
	U2TXREG  = c;
	
}


void serial_str( char *s)
{
	while(*s)
	{

		if (*s == '\n')
		{
			serial_ch(0x0d);
			serial_ch(0x0a);
		} else
			serial_ch(*s);
		s++;
	}
}



void serial_nib( unsigned char n)
{
	if (n >= 10)
		serial_ch(n - 10 + 'A');
	else
		serial_ch(n + '0');
}



void serial_hex( unsigned char ch)
{

	serial_nib(ch >> 4);
	serial_nib(ch & 0x0f);

	serial_ch(' ');
	serial_ch(' ');
}
*/



/* Compute checksum of packet */
unsigned int get_checksum(unsigned char *buff, unsigned int len)
{
	unsigned char ck = 0;
	
	while(len)
	{
		ck += buff[--len];
	}

	return(ck);
}
/* End of get_checksum */



/* Send data to host */
void serial_host_tx( unsigned char d )
{
	// Old polling method
	#if !BUFFER_HOST_TX
		while (U1STAbits.UTXBF);
		U1TXREG = d;
		return;
	#endif



	while(host_tx_pending > MAX_HOST_TX-2);

	interrupt_disable();


	// Add data to the buffer
	host_tx_buff[host_tx_head++] = d;

	// Another byte is now pending
	host_tx_pending++;
	
	// Wrap buffer
	if (host_tx_head  >= MAX_HOST_TX)
		host_tx_head = 0;


	// Force tx interrupt if the buffer was empty
	if (host_tx_pending == 1)
		IFS0bits.U1TXIF = 1; 


	interrupt_enable();
	
	return;


}
/* End of host send */


/* Send to host complete, send next byte if any */
inline void host_tx_complete( void )
{
	// Buffer empty, nothing to send
	if (host_tx_pending == 0) 
	{
		return;
	}
	
	// Refill the buffer
	//while (host_tx_pending && !U1STAbits.UTXBF)
	while (host_tx_pending && !U1STAbits.UTXBF)
	{

		// Send next byte
		U1TXREG = host_tx_buff[host_tx_tail++];		// Remove byte from buffer and send it

		host_tx_pending--;

		// Wrap buffer
		if (host_tx_tail  >= MAX_HOST_TX)
			host_tx_tail = 0;

	}
}
/* End of host_tx_complete */



/* Process byte from host */
void process_host_rx ( unsigned char d)
{

	if (serial_mode == MODE_WAIT)	// Waiting for device ID
	{
//.		packet_timeout = 0;

		device_id = d;

		cur_rx = 0;

		// select this serial channel
		if (device_id == DEVICE_CONT)	// Select camera 1
		{

	
	
		} else

		// select this serial channel
		if (device_id == DEVICE_CAM1)	// Select camera 1
		{

	
		} else
		if (device_id == DEVICE_CAM2) 	// Select camera 2
		{
			
		} 

		//	debug_ch(device_id);

		serial_mode = MODE_SIZE;  
	} else
	if (serial_mode == MODE_SIZE)	// Waiting for packet size
	{
		//debug("MS");
		packet_size = d;

		data_pending = packet_size;

		if (data_pending == 0)
			serial_mode = MODE_CHECK; // No data, keep reading this channel
		else
			serial_mode = MODE_DATA; // Need to load data	

	} else
	if (serial_mode == MODE_DATA) 	// Waiting for data size
	{
		//debug("MD");
		data_pending--;
	

		rx_buff[cur_rx++] = d;
	
		if (cur_rx >= RX_BUFF_SIZE)
		{
			cur_rx = RX_BUFF_SIZE-1;
			//cur_rx = 0;
			//controller_packet(rx_buff);

		}
		

		if (data_pending == 0)
		{
			serial_mode = MODE_CHECK; 
		}

	
	} else
	if (serial_mode == MODE_CHECK) 	// Waiting for checksum
	{
		//debug("MC");
		// = d;
		serial_mode = MODE_WAIT;

		msg_controller = 1;

		if (device_id == DEVICE_CONT)	// Local data, route here
		{
			controller_packet(rx_buff);
		} else
		if (device_id == DEVICE_CAM1)	// Route to camera 1
		{
		//	send_u2(d);
			//U2TXREG = d;
		} else
		if (device_id == DEVICE_CAM2) 	// Route to camera 1
		{
		//	send_u2(d);
//			U2TXREG = d;
		} 
	
	}

}
/* End of process_host_rx */



/* UART1 Serial RX Interrupt */
//
//
//  Main data from the host
void __attribute__((__interrupt__, auto_psv)) _U1RXInterrupt (void)
{
	IFS0bits.U1RXIF = 0;
	process_host_rx(U1RXREG & 0xff);
}
/* End of _U1RXInterrupt */



/* UART1 Serial TX Interrupt */
//
// 
void __attribute__((__interrupt__, auto_psv)) _U1TXInterrupt (void)
{	
	IFS0bits.U1TXIF = 0;			// Clear transmit interrupt flag
	host_tx_complete();
}
/* End of _U1TXInterrupt */



/* UART2 Serial RX Interrupt */
//
//
//
void __attribute__((__interrupt__, auto_psv)) _U2RXInterrupt (void)
{
	IFS1bits.U2RXIF = 0;

//	unsigned char d =  (U2RXREG) & 0xff;

//	U1TXREG = d;//tx_buff[0];
}
/* End of _U1RXInterrupt */



/* UART1 Serial TX Interrupt */
//
// 
void __attribute__((__interrupt__, auto_psv)) _U2TXInterrupt (void)
{
	IFS1bits.U2TXIF = 0;			// Clear transmit interrupt flag

//	if (u2_tx_head == u2_tx_tail) return;

//	U2TXREG = u2_tx_buff[u2_tx_tail++];
}
/* End of _U1TXInterrupt */



/* Reset systems for a serial timeout event */
void serial_timeout ( void )
{
	controller_reset();

	camera_reset();


	host_tx_tail = 0;
	host_tx_head = 0;
	host_tx_pending = 0;


	// Clear error flags
	U1STAbits.OERR = 0;
	U1STAbits.FERR = 0;

	U2STAbits.OERR = 0;
	U2STAbits.FERR = 0;


	//packet_timeout = 0;
	//update_timeout = 0;
	 debug_len = 0;
	 msg_debug = 0;

	camera_mode = 0;
}
/* End of serial timeout */





/* Update serial system */
void serial_check( void )
{

//	unsigned int error = 0;


	// This will latch once it reaches the timout, 
	if (update_timeout >= MAX_UPDATE_TIME)
	{
		if (update_timeout == MAX_UPDATE_TIME)
		{
			update_timeout = MAX_UPDATE_TIME + 1;// Only timeout once
			serial_timeout();
//			debug("Update Timeout");

//			serial_str("UT\n");

		}
		host_rx_error = 1;
	//	error  = 1;

	} else
	{
		update_timeout++;

	}




	if (update_timeout < MAX_UPDATE_TIME)
		host_rx_error = 0;


	
}
/* End of serial_check */



/* Setup serial */
void serial_init( void )
{


	U1BRG = (((FCY / HOST_BAUD) / 16) - 1);		// Create and set baud counter

	IEC0bits.U1RXIE = 1;
	IEC0bits.U1TXIE = 1;

//	U1STAbits.UTXISEL = 1;

	IFS0bits.U1RXIF = 0;	// Clear interrut flag

	U1MODEbits.UARTEN = 1;
	U1STAbits.UTXEN = 1;
	



	U2BRG = (((FCY / U2_BAUD) / 16) - 1);		// Create and set baud counter

	U2MODEbits.UARTEN = 1;
	U2STAbits.UTXEN = 1;
	U2STAbits.UTXISEL = 1;

	IFS1bits.U2RXIF = 0;	// Clear interrut flag
	IEC1bits.U2RXIE = 1;
	IEC1bits.U2TXIE = 1;



}
/* End of serial_init */


