/*
	Serial Camera Client 
	Version 11/24/2010-A
	(C) 2010 Jason Hunt nulluser@gmail.com

	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <windows.h>
#include <stdio.h>
#include <math.h>

#include "serial.h"
#include "display.h"
#include "utility.h"
#include "time.h"
//#include "math.h"
#include "main.h"

#include "input.h"

#include "automode.h"
#include "camera.h"

#include "serialcamera.h"

#include "controller.h"

#include "robot.h"



#define MODULE_NAME "Robot - "



bool remote_mode = false;


robot_class robot;  // Core instance of robot class


/* Robot constructor, set inital values */
robot_class::robot_class(void)
{
    // Setup Critical Sections
    InitializeCriticalSection(&control_data_lock);
    InitializeCriticalSection(&telemetry_data_lock);
    InitializeCriticalSection(&update_speed_lock);
    InitializeCriticalSection(&auto_mode_lock);
  
   
    
    telemetry_data.analog_data[0] = 0;
    telemetry_data.analog_data[1] = 0;
    telemetry_data.analog_data[2] = 0;
    telemetry_data.analog_data[3] = 0;
    
    
    update_speed = 0;
    updates = 0;

    auto_mode = false;

    reset_control();
}
/* End of robot_class constructor */

/* Stops motors and centers servos */
void robot_class::reset_control(void)
{
    control_type c;

    // Setup inital robot conditions

    c.left_motor_speed = 0;
    c.left_motor_target = 0;
    c.left_motor_dir = 0;

    c.right_motor_speed = 0;
    c.right_motor_target = 0;
    c.right_motor_dir = 0;

    c.servo_pos[0] = 0x80;
    c.servo_pos[1] = 0x80;

    set_control(c);
   
}
/* End of reset_control */


/* Robot destructor */
robot_class::~robot_class(void)
{
}
/* End of robot_class */


/* Start all robot sub systems */
void robot_class::start(void)
{
    
    controller_start();
        
//    int image_format = SERIAL_CAM_SIZE_160_120;
//    int image_format = SERIAL_CAM_SIZE_320_240;

        
//    camera.start(hwnd);

  serial_cam1 = new serial_cam_class("Left", 0x08, SERIAL_CAM_SIZE_160_120);
//    serial_cam2 = new serial_cam_class("Right", 0x18);

//    serial_cam1->start();

//    automode_start();
    
    
    
}
/* End of robot_class::start */


/* Stop all robot sub systems */
void robot_class::stop(void)
{
     log(MODULE_NAME "Stop");
 
 //   automode_stop();
    
 //   camera.stop();
    

    if (  serial_cam1 != NULL)
    {
      serial_cam1->stop();
      delete serial_cam1;
      }
    
    
    //stop_serial();          // Stop serial system
    
    controller_stop();
    
    
}
/* End of robot_class::stop */


/* Set robot auto mode */
void robot_class::set_auto_mode(bool mode)
{
    EnterCriticalSection(&auto_mode_lock);
    auto_mode = mode;    
    LeaveCriticalSection(&auto_mode_lock);
    
    if (!mode)
        reset_control();
    
}
/* End of set_auto_mode */


/* Set robot auto mode */
bool robot_class::get_auto_mode(void)
{
    bool mode;

    EnterCriticalSection(&auto_mode_lock);
    mode = auto_mode;
    LeaveCriticalSection(&auto_mode_lock);
    
    return(mode);
}
/* End of set_auto_mode */



/* Copy control data into the parameter */
// This consolidates all access to the control data to 
// Make it thread safe
//
void robot_class::get_control(control_type *control_data_copy)
{
    EnterCriticalSection(&control_data_lock);

    // Copy control data to copy
    memcpy(control_data_copy, &control_data, sizeof(control_data));

    LeaveCriticalSection(&control_data_lock);
}
/* End of get_control */


/* Put the control data into a buffer to be sent to the controller board */
void robot_class::get_control(unsigned char * buffer)
{

    EnterCriticalSection(&control_data_lock);

    // Clear buffer 
    memset(buffer, 0, CONTROL_SIZE);

    unsigned int i = 0;                                     // Index
    
  //  append_byte(buffer, i, 0xff);                           // Start byte
  
   /* unsigned char temp = 0;                                 // Temp control byte
    
    // Set direction bits      
    if (control_data.left_motor_dir) temp |= 0x01;          // Bit for left reverse
    if (control_data.right_motor_dir) temp |= 0x02;         // Bit for right reverse
      
    append_byte(buffer, i, temp);                           // Add the control byte
  
    // Speeds
    append_byte(buffer, i, (byte)control_data.left_motor_speed);  // Add left speed
    append_byte(buffer, i, (byte)control_data.right_motor_speed); // Add right speed

    // Servo positions
    append_byte(buffer, i, control_data.servo_pos[0]);      // Add pan
    append_byte(buffer, i, control_data.servo_pos[1]);      // Add tilt
     */                  
  
    int left1_base  = 0x80;
    int left2_base  = 0x80;
    
    int right1_base = 0x80;
    int right2_base = 0x80;
        
        
    if (control_data.left_motor_dir)
    {
        append_byte(buffer, i, left1_base + (byte)control_data.left_motor_speed);      // Add pan
        append_byte(buffer, i, left2_base + (byte)control_data.left_motor_speed);      // Add pan
    }
    else
    {
        append_byte(buffer, i, left1_base - (byte)control_data.left_motor_speed);      // Add pan    
        append_byte(buffer, i, left2_base - (byte)control_data.left_motor_speed);      // Add pan    
    }        
        
        
        
    if (control_data.right_motor_dir)        
    {
        append_byte(buffer, i, right1_base - (byte)control_data.right_motor_speed);      // Add tiltwwwwww
        append_byte(buffer, i, right2_base - (byte)control_data.right_motor_speed);      // Add tilt
    }
    else
    {
        append_byte(buffer, i, right1_base + (byte)control_data.right_motor_speed);      // Add tilt    
        append_byte(buffer, i, right2_base + (byte)control_data.right_motor_speed);      // Add tilt            
    }
      
          
   /* append_byte(buffer, i, 0x81);
    append_byte(buffer, i, 0x82);
    append_byte(buffer, i, 0x83);
    append_byte(buffer, i, 0x84);*/
    

    //add_line("control  %x  %x %x %x ", buffer[0], buffer[1], buffer[2], buffer[3]);


      //    buffer[SERIAL_SEND_SIZE-1] = 0xff;                      // End byte




   /* FILE *f = fopen("send.txt", "at");
    fprintf(f, "Send to robot %d %d %4f  %4f\n", control_data.left_motor_dir,
                                  control_data.right_motor_dir,
                                  control_data.left_motor_speed,
                                  control_data.right_motor_speed);

    fclose(f);*/





    LeaveCriticalSection(&control_data_lock);
}
/* End of get control */


/*  Copy passed control data to actual data */
void robot_class::set_control(control_type &new_control_data)
{
    EnterCriticalSection(&control_data_lock);

    // Copy new control data to actual control data
    memcpy(&control_data, &new_control_data, sizeof(control_type));

    LeaveCriticalSection(&control_data_lock);
}
/* End of set control */


/* Extract data from buffer into control data */
void robot_class::set_control(unsigned char *buffer)
{
    EnterCriticalSection(&control_data_lock);

    // Decode direction bits
    control_data.left_motor_dir = buffer[1] & 0x01;
    control_data.right_motor_dir = buffer[1] & 0x02;

    // Speeds
    control_data.left_motor_target = buffer[2];
    control_data.right_motor_target = buffer[3];

    if (control_data.left_motor_target == 0)
        control_data.left_motor_speed = 0;

    if (control_data.right_motor_target == 0)
        control_data.right_motor_speed = 0;



    // Servo Pos
    control_data.servo_pos[0] = buffer[4];
    control_data.servo_pos[1] = buffer[5];


   /*  FILE *f = fopen("send.txt", "at");
    fprintf(f, "sent from client %d %d %4f  %4f\n", control_data.left_motor_dir,
                                  control_data.right_motor_dir,
                                  control_data.left_motor_target,
                                  control_data.right_motor_target);

    fclose(f);*/

    
    LeaveCriticalSection(&control_data_lock);
}
/* End of set_control(buffer) */


/* Copy telemetry data into a struct */
void robot_class::get_telemetry(telemetry_type * telemetry_data_copy)   // telemetry into struct
{
    EnterCriticalSection(&telemetry_data_lock);

    // Copy telemetry data
    memcpy(telemetry_data_copy, &telemetry_data, sizeof(telemetry_type));

    LeaveCriticalSection(&telemetry_data_lock);    
}
/* End of get_telemetry */



/* Put telemetry data into buffer */
void robot_class::get_telemetry(unsigned char * buffer)   // tTlemetry into buffer
{
    EnterCriticalSection(&telemetry_data_lock);

    unsigned int i = 0;                                     // Index
    
    append_byte(buffer, i, 0xff);                           // Start byter
     
    // Add low words of analog
/*    append_byte(buffer, i, telemetry_data.analog_data[0] & 0xff);     
    append_byte(buffer, i, telemetry_data.analog_data[1] & 0xff);     
    append_byte(buffer, i, telemetry_data.analog_data[2] & 0xff);     
    append_byte(buffer, i, telemetry_data.analog_data[3] & 0xff);     

    append_byte(buffer, i, telemetry_data.analog_data[4] & 0xff);     
    append_byte(buffer, i, telemetry_data.analog_data[5] & 0xff);     
    append_byte(buffer, i, telemetry_data.analog_data[6] & 0xff);     
    append_byte(buffer, i, telemetry_data.analog_data[7] & 0xff);     */

    unsigned char temp = 0;                                 // Temp control byte
  
    // High words of analog
/*    temp |= (telemetry_data.analog_data[0] >> 8) & 0x03;
    temp |= ((telemetry_data.analog_data[1] >> 8) & 0x03) << 2;
    temp |= ((telemetry_data.analog_data[2] >> 8) & 0x03) << 4;
    temp |= ((telemetry_data.analog_data[3] >> 8) & 0x03) << 6;
                        
    append_byte(buffer, i, temp);       

    temp = 0;                           
  
    temp |= (telemetry_data.analog_data[4] >> 8) & 0x03;
    temp |= ((telemetry_data.analog_data[5] >> 8) & 0x03) << 2;
    temp |= ((telemetry_data.analog_data[6] >> 8) & 0x03) << 4;
    temp |= ((telemetry_data.analog_data[7] >> 8) & 0x03) << 6;
*/
    append_byte(buffer, i, temp); 
           
    append_byte(buffer, i, telemetry_data.encoder_counts[0] & 0xff);     
    append_byte(buffer, i, telemetry_data.encoder_counts[0] >> 8);     

    append_byte(buffer, i, telemetry_data.encoder_counts[1] & 0xff);     
    append_byte(buffer, i, telemetry_data.encoder_counts[1] >> 8);     

    append_byte(buffer, i, telemetry_data.encoder_counts[2] & 0xff);     
    append_byte(buffer, i, telemetry_data.encoder_counts[2] >> 8);     

    append_byte(buffer, i, telemetry_data.encoder_counts[3] & 0xff);     
    append_byte(buffer, i, telemetry_data.encoder_counts[3] >> 8);     
                        
    buffer[TELEMETRY_SIZE-1] = 0xff;                      // End byte
       
  /*  char b[300];
    sprintf(b, "data: %x %x %x %x %x %x %x %x", 
        buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], buffer[5], buffer[6], buffer[7]);
    add_line(b);
    */   
       
       
       
    LeaveCriticalSection(&telemetry_data_lock); 
}
/* End of get_telemetry (buffer) */



#define num_long_ave 50

double long_aves[num_long_ave];
double long_ave = 0;
double ave_offset = 0;


double vel = 0;
double pos = 0;

void reset_vals( void )
{
  ave_offset = long_ave;
 vel = 0;
pos = 0;   
     
}

void filter(float &val, float rc, float next, double dt)
{
    double alpha = dt / (rc + dt);

    val = alpha * next + (1 - alpha) * val;
}
    
    

/* Set telemeter data from buffer */
void robot_class::set_telemetry(unsigned char * buffer)   // telemetry into buffer
{
    EnterCriticalSection(&telemetry_data_lock);

    // Decode the compacted analog data
    //telemetry_data.analog_data[0] = buffer[1] + ((buffer[9] & 0xff) << 8 )  ;
    
    
    
    static double last_update = get_time();
    
    double tm = get_time();
    
    double dt = tm - last_update;
    
    last_update = tm;
    
        
//    add_line("dt: %f", dt);
    double rc = 0.5;

    double val;

        
    val = ((float)(buffer[0] + ((buffer[2] & 0x0f) << 8 )) / 4096.0  );

    filter(telemetry_data.analog_data[0], rc, val, dt);

    val = ((float)(buffer[1] + ((buffer[2] & 0xf0) << 4 )) / 4096.0  );

    filter(telemetry_data.analog_data[1], rc, val, dt);
         
                    
    
/*    telemetry_data.analog_data[1] = (buffer[2] + ((buffer[9]>>2)& 0x03) *256) ;
    telemetry_data.analog_data[2] = (buffer[3] + ((buffer[9]>>4)& 0x03) *256)  ;

    telemetry_data.analog_data[3] = (buffer[4] + ((buffer[9]>>6)& 0x03 )*256) ;
    telemetry_data.analog_data[4] = (buffer[5] + ((buffer[10])& 0x03) *256)  ;  
    telemetry_data.analog_data[5] = (buffer[6] + ((buffer[10]>>2)& 0x03) *256)  ;  

    telemetry_data.analog_data[6] = (buffer[7] + ((buffer[10]>>4)& 0x03) *256)  ;  
    telemetry_data.analog_data[7] = (buffer[8] + ((buffer[10]>>6)& 0x03) *256)  ;  
    
    
   
    
//     telemetry_data.encoders[0] = buffer[8];     
     
 
  */
  
        
/*      char b[100];    
  
     
     
     sprintf(b, "%x %x %x %x %x %x %x %x %x %x %x %x", buffer[1], buffer[2], buffer[3], buffer[4],
                                           buffer[5], buffer[6], buffer[7], buffer[8],
                                           buffer[9], buffer[10], buffer[11], buffer[12]
                                           );
     */
   //  add_line(b);   
     
     
     
     
//     sprintf(b, "last_time: %f  cur_time: %f diff: %f", last_time, get_time(), get_time() - last_time);
//     add_line(b);
         
                   
   
     
    
    LeaveCriticalSection(&telemetry_data_lock);     
}
/* End of set_telemetry (buffer) */


/* Apply new motion data to robot */
void robot_class::set_motion(double x_move, double y_move)
{
//    char b[400];
//    sprintf(b, "x_motion: %f  y_motion: %f", x_move, y_move);
//    add_line(b);

    double l_motor = 2*y_move + x_move;
    double r_motor = 2*y_move - x_move;
    
    if (l_motor > 1) l_motor = 1;
    if (l_motor < -1) l_motor = -1;

    if (r_motor > 1) r_motor = 1;
    if (r_motor < -1) r_motor = -1;

//    sprintf(b, "l_motor: %f  r_motor: %f", l_motor, r_motor);
//    add_line(b);

    control_type c;

    robot.get_control(&c);

    // Set motion values
    c.left_motor_target = fabs(l_motor) * 240; 
    c.right_motor_target = fabs(r_motor) * 240;

    c.left_motor_dir = l_motor < 0;
    c.right_motor_dir = r_motor < 0;  
    
    
    char b[300];
    
/*    sprintf(b, "l:  %f  r:  %f     ld: %d  rd:  %d", 
    
      c.left_motor_target,
    c.right_motor_target,

    c.left_motor_dir, 
    c.right_motor_dir);  
    
    
    
    debug(b);*/
    
    
    
    robot.set_control(c);
}
/* End of set_motion */


/* Return the robot update speed */
double robot_class::get_update_speed(void)
{
    EnterCriticalSection(&update_speed_lock);

    // Copy update speed
    double speed = update_speed;

    LeaveCriticalSection(&update_speed_lock);

    return(speed);
}
/* End of get_update_speed */


/* This is called when an update is done */
void robot_class::update_complete(bool fail)
{
    // Check for update fail
    if (fail)
    {
  //      updates = 0;    
//        update_speed = 0;
        return;
    }

    // Init on first run
    static double last_time = get_time();
    
    // Currect
    double cur_time = get_time();
    
    // Update sucess
    updates++;
    
    // Update speed 10 times per second
   /* if (cur_time - last_time > 0.1)
    {
        // Calculate update speed
        update_speed = updates / (cur_time - last_time);
        updates = 0;

        // Set new time offset
        last_time = cur_time;
    }*/
}
/* End of update_complete */


void robot_class::update( void)
{

    update_speed = updates;
        updates = 0;
}






