/*
	Serial Camera Client 
	Version 11/24/2010-A
	(C) 2010 Jason Hunt nulluser@gmail.com

	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


#include <windows.h>
#include <stdio.h>

#include "serial.h"
#include "display.h"
#include "main.h"
#include "controller.h"
#include "input.h"
#include "time.h"
#include "utility.h"
   #include "serialcamera.h"
#include "robot.h"

   
#define MODULE_NAME "Serial - "


serial_class serial;





serial_class:: serial_class( void )
{
    log(MODULE_NAME "Constructor");
	
	
	memset((void *)&ol_reader, 0, sizeof(OVERLAPPED));
	memset((void *)&ol_writer, 0, sizeof(OVERLAPPED));
	
		
    ol_reader.hEvent = CreateEvent(0, 1, 0, 0);
    ol_writer.hEvent = CreateEvent(0, 1, 0, 0);
	
    // Create thread stop event
    serial_stop = CreateEvent(0, 1, 0, 0);
	
//	flush_serial();
		
    serial_mutex = CreateMutex(NULL, FALSE, NULL); 
	

    h_serial = NULL;			      // Handle for serial port

    serial_ok = false;               // True if serial is working 

    timeout_flag = false;            // Set once for a timeout condition

    //char serial_port[10];                 // Filename of serial port

    enable_mux  = true;
    serial_debug = true;

    //HANDLE serial_mutex;

    //static HANDLE serial_stop;                 // Event to stop serial threads
    h_serial_thread = NULL;        // Handles for serial thread

    serial_running = true;
    debug_serial = 1;
    serial_hold = 0;

    //double flush_time;
                    

    packet_size = 0;        // Total size of packet, header and footer included
    b_index = 0;                 
    state = COMM_PACKET_WAIT;                 

}


serial_class::~serial_class( void )
{
    log(MODULE_NAME "Destructor");

}


    

void serial_class :: flush( void )
{
        add_line(MODULE_NAME "Enter Flush");

        state = COMM_PACKET_FLUSH ;

        flush_time = get_time() + 1;

}



int serial_class :: validate_packet(unsigned char *buff, unsigned int len)
{

    int c = get_checksum(buff, len-1);
    

    if (c != buff[len-1])
    {
       
        add_line("Invalid packet checksum actual: %02x  Expected: %02x", buff[len-1], c);
        
        show_buffer("Buffer: ", buff, len, 0);
        
        return(-1);
    }
    else
    {
        return(0);
    }
}



int serial_class :: process_packet( unsigned char *buff, unsigned int len )
{
    if (validate_packet(buff, len))
    {
        // Force the controller to timeout and flush remaining
        flush();
    
        add_line(MODULE_NAME "Unable to validate packet");
        return(1);
    }
    else
    {
        unsigned char device_id = buff[1];

        // dispatch the packet
        if (device_id == DEVICE_CONT)
            controller_data(buff + HEADER_SIZE, len - HEADER_SIZE  - FOOTER_SIZE);
        else       
        if (device_id == DEVICE_CAM1)
            robot.serial_cam1->camera_data(buff + HEADER_SIZE, len - HEADER_SIZE  - FOOTER_SIZE);
        else       
          
        if (device_id == DEVICE_DEBUG)
        {
            buff[len - FOOTER_SIZE] = 0;
                
            add_line((char *)(buff + HEADER_SIZE));
            
        }
        else
            add_line(MODULE_NAME "Data for invalid device: %d", device_id);
            
    }
    
    return(0);
}




    
    
int  serial_class :: process_byte(unsigned char b)
{



    if (state == COMM_PACKET_FLUSH) 
    {
        log(MODULE_NAME "Flush Ignore: %x", b);
        return(1);
    }


//    if (debug_serial)
  //      log(MODULE_NAME "Process byte: %x (%c) State: %d pending: %d  Bindex: %d", b, b,state,packet_size,  b_index);



    if (state == COMM_PACKET_WAIT)      // Waiting for packet start
    {
        b_index = 0;

        packet_size = b;
     
        buff[b_index++] = b;
        
        if (packet_size < 4)
        {
            add_line(MODULE_NAME "Invalid packet size");
        
            state = COMM_PACKET_WAIT;   
            return(1);                 
        }
        state = COMM_PACKET_DATA;                    
    
    } else

    if (state == COMM_PACKET_DATA)      // Getting packet data
    {
        if (b_index >= COMM_RX_MAX)
        {
            state = COMM_PACKET_WAIT;
            add_line(MODULE_NAME "Process byte buffer size exceeded");
            return(1);
        }
    
        buff[b_index++] = b;
    
        // We have the entire packer, process it
        if (b_index == packet_size)
        {
            state = COMM_PACKET_WAIT;    

            if (process_packet( buff, packet_size ))
            return(1);

        }
    }
    return(0);
}




/* preform a transfer with the controller */
int serial_class :: serial_transfer(unsigned char device_id,
                    unsigned char *send_buff, int send_bytes,  
                    unsigned char *recv_buff, int recv_bytes)
{
}
/* End of serial_transfer */



int serial_class :: serial_send(unsigned char *buff, unsigned int len)
{

    HANDLE comm_handles[2] = {serial_stop, ol_writer.hEvent};

    DWORD sent;

    WriteFile(h_serial, buff, len, &sent, &ol_writer);

//log(MODULE_NAME "wait send");
    int ret = WaitForMultipleObjects (2, comm_handles, 0, INFINITE);

//log(MODULE_NAME "return");
	if (ret == WAIT_TIMEOUT)
    {
    	add_line(MODULE_NAME "Write timeout error: %d", GetLastError());
         
     //   return(false);
    } 
	
    // Thread termination
    if (ret == WAIT_OBJECT_0)
    {
        CloseHandle(ol_reader.hEvent);
        return(true);
    } 
        
	if (ret == WAIT_OBJECT_0+1)
    {  
        if (GetOverlappedResult(h_serial, &ol_writer, &sent, FALSE))
        {
            //show_buffer("Send ", buff, sent, 0);
            
            }
            
    }
	
    //CloseHandle(ol_writer.hEvent);

	
    return(false);
    
}




/* Preform a transfer with the controller */
int serial_class :: do_serial_packet(unsigned char device_id, 
                     unsigned char *send_buff, 
                     int send_bytes)
{

    if (send_bytes <= 0) return(0);
    
    unsigned char data[HEADER_SIZE] = {device_id, send_bytes};
    
    // Send packet header
    if (serial_send (data, HEADER_SIZE) == -1) return(1);

    // Send packet data
    if (serial_send (send_buff, send_bytes) == -1) return(1);

    // Set checksum
    data[0] =  get_checksum (send_buff, send_bytes + 2);

    // Send packet footer
    if (serial_send (data, FOOTER_SIZE) == -1) return(1);
              
    return(0);
}
/* End of do_serial_packet */
  
    
    
/* Send a packet over serial to the controller */
int serial_class :: serial_packet(unsigned char device_id, 
                  unsigned char *send_buff, 
                  int send_bytes)
{
   // log(MODULE_NAME " serial packet start waiting: %d", device_id);


    if (state == COMM_PACKET_FLUSH) return(0);

    HANDLE event_handles[] = {serial_stop, serial_mutex};

int   	ret = WaitForMultipleObjects (2, event_handles, 0, INFINITE);

   // DWORD ret = WaitForSingleObject(serial_mutex, INFINITE);

  // log(MODULE_NAME " serial packet start. Device: %d", device_id);
    
    // Could not get the mutex
    if (ret == WAIT_TIMEOUT)
    {
        log(MODULE_NAME "serial_packet - Unable to get mutex");
        return(1);
    } else
    	if (ret == WAIT_OBJECT_0)         // Thread Exit
    	{
           
        } else
        	if (ret == WAIT_OBJECT_0+1)
    	{

        ret = do_serial_packet(device_id, send_buff, send_bytes);


        } 
	
	

                          
 //  log(MODULE_NAME " transfer end. Device: %d", device_id);

    ReleaseMutex(serial_mutex); 

    return(ret);
}
/* End of serial_transfer */

    
  
  
  
  
/* Serial thread for rx processing */
DWORD WINAPI serial_class :: serial_thread( void )
{
    bool done = false;

    unsigned int max = 8192;
    unsigned char buff[max];
    
    SetCommMask(h_serial, EV_RXCHAR);
        
    HANDLE event_handles[] = {serial_stop, ol_reader.hEvent};
    
    unsigned int index = 0;


    log(MODULE_NAME " Enter Serial Thread"); 

    while(1)
    {   
    
      index = 0; 
        DWORD mask;

        DWORD ret = WaitCommEvent(h_serial, &mask, &ol_reader) ;
        
        //log(MODULE_NAME " COmm mask: %d", ret); 
        
        
        if (!ret)
        {
            if (GetLastError () != ERROR_IO_PENDING)
                log(MODULE_NAME " Wait Comm return not IO pending: %d", GetLastError ());
        }
 
        // Wait for thread exit, event, or timeout
    	ret = WaitForMultipleObjects (2, event_handles, 0, TIMEOUT);

        // Did not get a packet in the timout period
        if (ret == WAIT_TIMEOUT)
        {
        	log(MODULE_NAME "Read timeout");

        } else

    	if (ret == WAIT_OBJECT_0)         // Thread Exit
    	{
            return(0);
        } else
	

        if (ret == WAIT_OBJECT_0+1) // Read complete, deal with the data
        {  
            // See what happened
            if (GetCommMask(h_serial, &mask) )
            {
                if ( mask & EV_RXCHAR )
                {
                    DWORD num_read;
                    bool reading = true;
                    index = 0;
                unsigned int t = 1000;
                    do
                    {
                        ReadFile(h_serial, buff+index, 1, &num_read, &ol_reader);
                        //ResetEvent( ol_reader.hEvent  );
                        index += num_read;    
                    }
//                    while (t--);
                    while(num_read > 0 && index < max-1);
                
//                        if (num_read > 0)  
                        //process_byte(buff[0]);
                }         
            

            }


            if (index > 0)
            {
                for (int i = 0; i < index; i++)
                    if ( process_byte(buff[i]) ) break;                    
            }
   



        } 
       ResetEvent ( ol_reader.hEvent );
    }

  log(MODULE_NAME " Leave Serial Thread"); 



}
/* End of serial thread */


/* Wrapper for the camera thread */
DWORD WINAPI serial_class::thread_wrapper(LPVOID pvoid) 
{
    serial_class *s = (serial_class *) pvoid;
    
    s->serial_thread();

	return (DWORD)1;
}
/* End of serial_cam_class */






/* Starts the serial system */
void serial_class :: start( unsigned int baud)
{
	add_line(MODULE_NAME "Start");

	serial_ok = true;
  
/*	h_serial = CreateFile(serial_port,							// Port name 
			 			  GENERIC_READ | GENERIC_WRITE,		// File access
		                  0,								// share mode
			 			  NULL,								// Security
			 			  OPEN_ALWAYS,						// Creation					
			 			  0,								// Flags
						  NULL );							// Template
*/

	h_serial = CreateFile(serial_port,				        // Port name 
			 			  GENERIC_READ | GENERIC_WRITE,		// File access
		                  0,								// share mode
			 			  0,								// Security
                          OPEN_EXISTING,            
                          FILE_FLAG_OVERLAPPED,  
						  0 );	




	if (h_serial == INVALID_HANDLE_VALUE)
	{
        log(MODULE_NAME "Unable to open serial port (%s)", serial_port);
    	serial_ok = false;
    
		return;
	}
	
    if(!SetupComm(h_serial, 8192, 8192))
    {
		log(MODULE_NAME "Unable to set buffers");
    	serial_ok = false;

		return;
    }
	

	DCB serial_param;

	if (!GetCommState(h_serial, &serial_param))
	{
		log(MODULE_NAME "Unable to get commstate");
    	serial_ok = false;

		return;
	}

    serial_param.BaudRate = baud;
	serial_param.fParity = false;	
    serial_param.fBinary  = true;
	serial_param.fOutxCtsFlow = false;
	serial_param.fOutxDsrFlow = false;
	serial_param.fDtrControl = DTR_CONTROL_ENABLE;
    serial_param.fDsrSensitivity = false;
    serial_param.fErrorChar = false;
    serial_param.fNull = false;
	serial_param.fOutX = false;
	serial_param.fInX = false;
    serial_param.fRtsControl = RTS_CONTROL_ENABLE;		// Disable request to send control
    serial_param.Parity = NOPARITY;
    serial_param.ByteSize = 8;
	serial_param.StopBits = ONESTOPBIT;

	if (!SetCommState(h_serial, &serial_param))
	{
		add_line(MODULE_NAME "Unable to set commstate");
       	serial_ok = false;

		return;
	}

	COMMTIMEOUTS timeouts;

/*	timeouts.ReadIntervalTimeout = 5;
	
	timeouts.ReadTotalTimeoutMultiplier = 1;
	timeouts.ReadTotalTimeoutConstant = 5; 
	timeouts.WriteTotalTimeoutMultiplier = 1;
	timeouts.WriteTotalTimeoutConstant = 500;*/


	timeouts.ReadIntervalTimeout = MAXDWORD;
	
	timeouts.ReadTotalTimeoutMultiplier = 0;
	timeouts.ReadTotalTimeoutConstant = 0; 
	timeouts.WriteTotalTimeoutMultiplier = 0;
	timeouts.WriteTotalTimeoutConstant = 0;


	if (!SetCommTimeouts(h_serial, &timeouts))
	{
		add_line(MODULE_NAME "Unable to get comm timeouts");
    	serial_ok = false;
		
		return;
	}
	
	

    DWORD id;    // Dummy thread ID
    
    
        h_serial_thread  = CreateThread (NULL, 0, 
                                     (unsigned long (__stdcall *)(void *))thread_wrapper, 
                                     (void *)this, 0, &id);
    
    
    
    
//	h_serial_thread = CreateThread(0, 0, serial_thread, NULL, 0, &id);
	
	add_line(MODULE_NAME "Online");
}
/* End of start serial */



/* Close the serial port */
void serial_class :: stop( void )
{    
	add_line(MODULE_NAME "Stop");

//   serial_running = false;

    SetEvent(serial_stop);      // Stop serial threads

      log(MODULE_NAME "Event Sig");

    
    // Wait for threads to exit
    WaitForSingleObject(h_serial_thread, INFINITE);
    
      log(MODULE_NAME "Thread return");
    
    CloseHandle(h_serial);      // Close the serial port
   
    CloseHandle(ol_reader.hEvent);
    CloseHandle(ol_writer.hEvent);
   

    if (h_serial != NULL)
    	CloseHandle(h_serial);
    	
    log(MODULE_NAME "Stop Complete");    	
    	
}
/* End of stop serial */




int serial_class :: get_state( void )
{
    return ( serial_ok);    
}

/*
void serial_class :: serial_node_baud(unsigned int baud)
{
    unsigned int constant = (unsigned int)(baud  * 0.004096);

    log("Setting node baud con %d", constant);

    char buff[100];

    sprintf(buff, "+++%c",0x0d);
    serial_send((unsigned char *)buff, strlen(buff));
  //  serial_read((unsigned char *)buff, 6);

    sprintf(buff, "atsw20,%d,0,0,1%c", constant, 0x0d);
    serial_send((unsigned char *)buff, strlen(buff));
//    serial_read((unsigned char *)buff, 6);

    sprintf(buff, "atmd%c", 0x0d);
    serial_send((unsigned char *)buff, strlen(buff));
  //  serial_read((unsigned char *)buff, 6);

   

}
*/






void serial_class :: check ( void )
{


    if (state == COMM_PACKET_FLUSH)
    {
        if (get_time() > flush_time)
        {
            state = COMM_PACKET_WAIT;
            add_line(MODULE_NAME "Exit Flush");
        }
        else
            return;
    }



}

















