/*
	Serial Camera Client 
	Version 11/24/2010-A
	(C) 2010 Jason Hunt nulluser@gmail.com

	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


#include <windows.h>
#include <stdio.h>

#include "serial.h"
#include "display.h"
#include "main.h"
#include "decompress.h"
#include "serialcamera.h"
#include "utility.h"
#include "time.h"


#define MODULE_NAME "Serial Cam"


/* Serial Camera constructor */
serial_cam_class::serial_cam_class( char * name_,  unsigned char id, unsigned int format)
{
    log(MODULE_NAME " [%s - %d] - Construct", name_, id);

    image_format = format;


    strcpy(name, name_);
    device_id = id;

    memdc = NULL;	           	           // Memory device context
    h_mem_bitmap = NULL;                // Offscreen bitmap


    display_x = 640;
    display_y = 480;
    
    
      buff_index = 0;
      image_size = 0;
      
    start();
}
/* End of serial_cam_class constructor */

/* Serial_cam_class destructor */
serial_cam_class::~serial_cam_class( void )
{
    log(MODULE_NAME " [%s - %d] - Destruct", name, device_id);

   // stop();
}
/* End of serial_cam_class deconstructor */



    
/* Setup the image buffer */
void serial_cam_class::camera_setup_image(int image_size)
{
    log(MODULE_NAME " [%s - %d] - Setup Image", name, device_id);

    if (image_size == SERIAL_CAM_SIZE_160_120)
    {
        x_size = 160;
        y_size = 120;
    } else
    if (image_size == SERIAL_CAM_SIZE_320_240)
    {
        x_size = 320;
        y_size = 240;
    } else
    if (image_size == SERIAL_CAM_SIZE_640_480)
    {
        x_size = 640;
        y_size = 480;
    } 

  if (camera_buffer != NULL) free(camera_buffer);    

    camera_buffer = (color_type *) malloc(x_size * y_size * sizeof(color_type));
        
    if (camera_buffer == NULL)
    {            
        log(MODULE_NAME " [%s - %d] - Unable to create camera buffer", name, device_id);
        return;
    }        

    if (compressed_buffer != NULL) free(compressed_buffer);    

    max_compressed = x_size * y_size * sizeof(color_type) * 2;

    compressed_buffer = (byte *) malloc(max_compressed);
        
    if (compressed_buffer == NULL)
    {            
        log(MODULE_NAME " [%s - %d] - Unable to create compressed buffer", name, device_id);
        return;
    }        


    if (transcode_buffer != NULL) free(transcode_buffer);    

    transcode_buffer = (byte *) malloc(x_size * y_size * sizeof(color_type));
        
    if (transcode_buffer == NULL)
    {            
        log(MODULE_NAME " [%s - %d] - Unable to create transcode buffer", name, device_id);
        return;
    } 

    // Set the new Bitmap information
    camera_buffer_info.bmiHeader.biSize = sizeof(camera_buffer_info.bmiHeader);
    camera_buffer_info.bmiHeader.biWidth = x_size;
    camera_buffer_info.bmiHeader.biHeight = y_size; 
    camera_buffer_info.bmiHeader.biPlanes = 1;
    camera_buffer_info.bmiHeader.biBitCount = 32;
    camera_buffer_info.bmiHeader.biCompression = BI_RGB;
    camera_buffer_info.bmiHeader.biSizeImage = x_size * y_size  * 4;
    camera_buffer_info.bmiHeader.biClrUsed = 0;
    camera_buffer_info.bmiHeader.biClrImportant = 0;



    // Get main DC
    HDC hdc = GetDC(main_win);

  	// Create Memory device contextand bitmap
    memdc = CreateCompatibleDC(hdc);	

    // Create off screen buffer
  	h_mem_bitmap = CreateCompatibleBitmap(hdc, display_x, display_y);

  	// Select the off screen buffer into the DC
   	SelectObject(memdc, h_mem_bitmap);		

    ReleaseDC(main_win, hdc);       // Main DC is no longer needed



//    image_start();
}
/* End of camera_setup_image */



/* Copy offscreen buffer to display */
void serial_cam_class::paint( void )
{
    PAINTSTRUCT ps;
    HDC hdc = BeginPaint(cam_window, &ps);
   // log("Serial cam paint");
    
    StretchDIBits(memdc, 0, 0, display_x, display_y, 
                  0, 0, x_size, y_size, 
                  camera_buffer, &camera_buffer_info,
                  DIB_RGB_COLORS, SRCCOPY);
      
        
    BitBlt(hdc, 0, 0, display_x, display_y, memdc, 0, 0, SRCCOPY);
    EndPaint(cam_window, &ps);  
}
/* End of do_paint */


/* Wrapper for the camera window procedure */
LRESULT CALLBACK serial_cam_class::cam_proc_wrapper(HWND hwnd, UINT message, WPARAM w, LPARAM l)
{
    // Get object pointer from window data and call actual wndproc
        
    // Set the class pointer on creation
    if (message == WM_NCCREATE)
    {     
        if(l)
        {
            LONG handle = (LONG)((CREATESTRUCT*)l)->lpCreateParams;
        
            if(handle)
                SetWindowLong(hwnd, GWL_USERDATA, handle);
        }
    }
    
    serial_cam_class *cam = (serial_cam_class*)::GetWindowLong(hwnd, GWL_USERDATA);
            
    // Call the member function     
    cam -> cam_proc(hwnd, message, w, l);
}
/* End of cam_proc_wrapper */


/* Ehe message handler */
LRESULT CALLBACK serial_cam_class::cam_proc(HWND hwnd, UINT message, WPARAM w, LPARAM l)
{
    switch (message)                 
    {
        case WM_CREATE:
            cam_window = hwnd;      
            return(0);

        case WM_PAINT:      
            paint();    
            return(0);

        case WM_TIMER:      return(0);
        case WM_DESTROY:    return(0);
        case WM_CHAR:       return(0);
        case WM_KEYDOWN:    return(0);
        case WM_KEYUP:      return(0);
        
        case WM_SIZE:       return(0);

        default: return DefWindowProc (hwnd, message, w, l);
    }

    return(0);
}
/* End of winproc */





/* Starts the camera system */
void serial_cam_class::start( void )
{
    log(MODULE_NAME " [%s - %d] - Start", name, device_id);

    if (camera_running) return;

    InitializeCriticalSection(&camera_thread_lock);

    camera_running = true;


    /* Create the camera window */

    WNDCLASS wnd_class;

    char class_name[] = "SerialCamWin";
       
    wnd_class.hInstance = main_hinst;
    wnd_class.lpszClassName = class_name;
    wnd_class.lpfnWndProc = cam_proc_wrapper;
    wnd_class.style = 0;
    
    wnd_class.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wnd_class.hCursor = LoadCursor(NULL, IDC_ARROW);
    wnd_class.lpszMenuName = NULL;
    
    wnd_class.cbClsExtra = 0;
    wnd_class.cbWndExtra = 0;
    
    wnd_class.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);

    // Only register the class if we have not already
    if (FindWindow(class_name, NULL) == NULL)
        if (!RegisterClass(&wnd_class)) 
        {
            log(MODULE_NAME " [%s - %d] - Start - Unable to register window class", name, device_id);
            return;
        }
        
    // Create the window
    // Need to pass this as a param for the wrapper
    CreateWindowEx(0, class_name, "Serial Camera", 
                             WS_CAPTION  | WS_SYSMENU | WS_MINIMIZEBOX ,
                             200, 200,   // Pos
                             display_x, display_y+20,   // Size
                             HWND_DESKTOP, NULL, main_hinst, this); 

    ShowWindow(cam_window, SW_SHOWDEFAULT);
    UpdateWindow(cam_window);
   
     camera_setup_image(image_format);
   
   /*
    h_camera_thread  = CreateThread (NULL, 0, 
                                     (unsigned long (__stdcall *)(void *))thread_wrapper, 
                                     (void *)this, 0, &camera_thread_id);*/
}
/* End of camera_start */


/* Stop the camera */
void serial_cam_class::stop ( void )
{
    log(MODULE_NAME " [%s - %d] - Stop", name, device_id);


}
/* End of camera stop */    
    
    
    




void serial_cam_class::camera_data( unsigned char *buff, int len)
{
static double last_update = get_time();
static unsigned int updates = 0;

    if (buff[0] == 5)
    {
        image_size = buff[1]*256 + buff[2];

        add_line("Cam size: %d", image_size);
    
          show_buffer("Cam data", buff, len, 0);


    
        buff_index = 0;
    
    }else
    if (buff[0] == 6)
    {
        if (image_size > 0)
        {
            for (int i = 0; i < len-1; i++)
                compressed_buffer[buff_index++] = buff[i+1];

            show_buffer("Cam data", buff+1, len-1, 0);


   
            if (buff_index >= image_size)
            {
                updates++;
                
                if (updates >= 2)
                {
                    update_rate = updates / (get_time() - last_update);
                    last_update = get_time();
                    updates = 0;
                }
            
            
                decompress_image((char *)compressed_buffer, image_size, camera_buffer, x_size, y_size);
                InvalidateRect (cam_window, NULL, FALSE);
                add_line("Image Complete");
            }
        }
    }
    if (buff[0] == 7)
    {
    
     add_line("Camera Message: %d", buff[1]);
    
    }
        
    
}







